# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_ncd_arith.

This module contains unit tests for abydos.distance.NCDarith
"""

import unittest

from abydos.compression import Arithmetic
from abydos.distance import NCDarith

from .. import NIALL


class NCDarithTestCases(unittest.TestCase):
    """Test compression distance functions.

    abydos.distance.NCDarith
    """

    arith = Arithmetic(' '.join(NIALL))
    cmp = NCDarith()
    cmp_probs = NCDarith(arith.get_probs())

    def test_ncd_arith_dist(self):
        """Test abydos.distance.NCDarith.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp_probs.dist('', ''), 0)
        self.assertGreater(self.cmp.dist('a', ''), 0)
        self.assertGreater(self.cmp_probs.dist('a', ''), 0)
        self.assertGreater(self.cmp.dist('abcdefg', 'fg'), 0)

        self.assertAlmostEqual(
            self.cmp_probs.dist('Niall', 'Neil'), 0.608695652173913
        )
        self.assertAlmostEqual(
            self.cmp_probs.dist('Neil', 'Niall'), 0.608695652173913
        )
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Neil'), 0.6875)
        self.assertAlmostEqual(self.cmp.dist('Neil', 'Niall'), 0.6875)
        self.assertAlmostEqual(
            self.cmp_probs.dist('Njáll', 'Njall'), 0.714285714285714
        )
        self.assertAlmostEqual(
            self.cmp_probs.dist('Njall', 'Njáll'), 0.714285714285714
        )
        self.assertAlmostEqual(self.cmp.dist('Njáll', 'Njall'), 0.75)
        self.assertAlmostEqual(self.cmp.dist('Njall', 'Njáll'), 0.75)

    def test_ncd_arith_sim(self):
        """Test abydos.distance.NCDarith.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp_probs.sim('', ''), 1)
        self.assertLess(self.cmp.sim('a', ''), 1)
        self.assertLess(self.cmp_probs.sim('a', ''), 1)
        self.assertLess(self.cmp.sim('abcdefg', 'fg'), 1)

        self.assertAlmostEqual(
            self.cmp_probs.sim('Niall', 'Neil'), 0.3913043478260869
        )
        self.assertAlmostEqual(
            self.cmp_probs.sim('Neil', 'Niall'), 0.3913043478260869
        )
        self.assertAlmostEqual(self.cmp.sim('Niall', 'Neil'), 0.3125)
        self.assertAlmostEqual(self.cmp.sim('Neil', 'Niall'), 0.3125)
        self.assertAlmostEqual(
            self.cmp_probs.sim('Njáll', 'Njall'), 0.285714285714285
        )
        self.assertAlmostEqual(
            self.cmp_probs.sim('Njall', 'Njáll'), 0.285714285714285
        )
        self.assertAlmostEqual(self.cmp.sim('Njáll', 'Njall'), 0.25)
        self.assertAlmostEqual(self.cmp.sim('Njall', 'Njáll'), 0.25)


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_waahlin.

This module contains unit tests for abydos.phonetic.Waahlin
"""

import unittest

from abydos.phonetic import Soundex, Waahlin


class WaahlinTestCases(unittest.TestCase):
    """Test Wåhlin functions.

    test cases for abydos.phonetic.Waahlin
    """

    pa = Waahlin()
    pa_sdx = Waahlin(Soundex())

    def test_waahlin(self):
        """Test abydos.phonetic.Waahlin."""
        self.assertEqual(self.pa.encode(''), '')

        self.assertEqual(self.pa.encode('kjol'), '+OL')
        self.assertEqual(self.pa.encode('stråken'), 'STRÅ+EN')
        self.assertEqual(self.pa.encode('skytten'), '*YTTEN')
        self.assertEqual(self.pa.encode('ljuden'), 'JUDEN')
        self.assertEqual(self.pa.encode('högre'), 'HÖGRE')
        self.assertEqual(self.pa.encode('först'), 'FÖRST')
        self.assertEqual(self.pa.encode('hval'), 'VAL')
        self.assertEqual(self.pa.encode('hrothgar'), 'ROTHGAR')
        self.assertEqual(self.pa.encode('denna'), 'DENNA')
        self.assertEqual(self.pa.encode('djur'), 'JUR')
        self.assertEqual(self.pa.encode('hjärta'), 'JERTA')
        self.assertEqual(self.pa.encode('STIEN'), '*EN')
        self.assertEqual(self.pa.encode('SKJERN'), '*ERN')
        self.assertEqual(self.pa.encode('HIELPA'), 'JELPA')
        self.assertEqual(self.pa.encode('CEILA'), 'SEILA')
        self.assertEqual(self.pa.encode('GELD'), 'JELD')
        self.assertEqual(self.pa.encode('IERN'), 'JERN')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('kjol'), 'ÇOL')
        self.assertEqual(self.pa.encode_alpha('stråken'), 'STRÅÇEN')
        self.assertEqual(self.pa.encode_alpha('skytten'), 'ŠYTTEN')
        self.assertEqual(self.pa.encode_alpha('ljuden'), 'JUDEN')

    def test_waahlin_soundex(self):
        """Test abydos.phonetic.Waahlin with Soundex."""
        self.assertEqual(self.pa_sdx.encode(''), '')

        self.assertEqual(self.pa_sdx.encode('kjol'), '+O400')
        self.assertEqual(self.pa_sdx.encode('stråken'), 'ST625')
        self.assertEqual(self.pa_sdx.encode('skytten'), '*Y350')
        self.assertEqual(self.pa_sdx.encode('ljuden'), 'JU350')
        self.assertEqual(self.pa_sdx.encode('högre'), 'HO260')
        self.assertEqual(self.pa_sdx.encode('först'), 'FO623')
        self.assertEqual(self.pa_sdx.encode('hval'), 'VA400')
        self.assertEqual(self.pa_sdx.encode('hrothgar'), 'RO326')
        self.assertEqual(self.pa_sdx.encode('denna'), 'DE500')
        self.assertEqual(self.pa_sdx.encode('djur'), 'JU600')
        self.assertEqual(self.pa_sdx.encode('hjärta'), 'JA630')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_sift4_simplest.

This module contains unit tests for abydos.distance.Sift4Simplest
"""

import unittest

from abydos.distance import Sift4Simplest


class Sift4TestCases(unittest.TestCase):
    """Test Sift4Simplest functions.

    abydos.distance.Sift4Simplest
    """

    cmp = Sift4Simplest()

    def test_sift4_simplest_dist_abs(self):
        """Test abydos.distance.Sift4Simplest.dist_abs."""
        # tests copied from Lukas Benedix's post at
        # https://siderite.blogspot.com/2014/11/super-fast-and-accurate-string-distance.html
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 3)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 3)

        self.assertEqual(self.cmp.dist_abs('a', 'a'), 0)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0)

        self.assertEqual(self.cmp.dist_abs('a', 'ab'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'abc'), 1)
        self.assertEqual(self.cmp.dist_abs('abcdefg', 'xabxcdxxefxgx'), 10)

        self.assertEqual(self.cmp.dist_abs('ab', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('xabxcdxxefxgx', 'abcdefg'), 10)

        self.assertEqual(self.cmp.dist_abs('a', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'bc'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'axc'), 1)
        self.assertEqual(
            self.cmp.dist_abs('xabxcdxxefxgx', '1ab2cd34ef5g6'), 6
        )

        self.assertEqual(self.cmp.dist_abs('example', 'samples'), 2)
        self.assertEqual(self.cmp.dist_abs('sturgeon', 'urgently'), 4)
        self.assertEqual(self.cmp.dist_abs('levenshtein', 'frankenstein'), 10)
        self.assertEqual(self.cmp.dist_abs('distance', 'difference'), 7)

        # Tests copied from
        # https://github.com/tdebatty/java-string-similarity/blob/master/src/test/java/info/debatty/java/stringsimilarity/experimental/Sift4Test.java
        self.assertEqual(
            Sift4Simplest(5).dist_abs(
                'This is the first string', 'And this is another string'
            ),
            13,
        )
        self.assertEqual(
            Sift4Simplest(10).dist_abs(
                'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
                'Amet Lorm ispum dolor sit amet, consetetur adixxxpiscing'
                + ' elit.',
            ),
            20,
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_stiles.

This module contains unit tests for abydos.distance.Stiles
"""

import unittest

from abydos.distance import Stiles


class StilesTestCases(unittest.TestCase):
    """Test Stiles functions.

    abydos.distance.Stiles
    """

    cmp = Stiles()
    cmp_no_d = Stiles(alphabet=0)

    def test_stiles_sim(self):
        """Test abydos.distance.Stiles.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.760700314616495)
        self.assertEqual(self.cmp.sim('', 'a'), 0.760700314616495)
        self.assertEqual(self.cmp.sim('abc', ''), 0.7416916584588271)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.7416916584588271)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.4768516719017855)

        self.assertAlmostEqual(self.cmp.sim('Nigel', 'Niall'), 0.5744293838)
        self.assertAlmostEqual(self.cmp.sim('Niall', 'Nigel'), 0.5744293838)
        self.assertAlmostEqual(self.cmp.sim('Colin', 'Coiln'), 0.5744293838)
        self.assertAlmostEqual(self.cmp.sim('Coiln', 'Colin'), 0.5744293838)
        self.assertAlmostEqual(
            self.cmp.sim('ATCAACGAGT', 'AACGATTAG'), 0.5909028826
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.sim('', ''), 1.0)
        self.assertAlmostEqual(self.cmp_no_d.sim('a', ''), 0.9511587063686434)
        self.assertAlmostEqual(self.cmp_no_d.sim('', 'a'), 0.9511587063686434)
        self.assertAlmostEqual(
            self.cmp_no_d.sim('abc', ''), 0.9309340273884292
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim('', 'abc'), 0.9309340273884292
        )
        self.assertEqual(self.cmp_no_d.sim('abc', 'abc'), 1.0)
        self.assertAlmostEqual(
            self.cmp_no_d.sim('abcd', 'efgh'), 0.47481536969259386
        )

        self.assertAlmostEqual(
            self.cmp_no_d.sim('Nigel', 'Niall'), 0.5216609379
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim('Niall', 'Nigel'), 0.5216609379
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim('Colin', 'Coiln'), 0.5216609379
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim('Coiln', 'Colin'), 0.5216609379
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim('ATCAACGAGT', 'AACGATTAG'), 0.5532905837
        )

    def test_stiles_dist(self):
        """Test abydos.distance.Stiles.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0.0)
        self.assertEqual(self.cmp.dist('a', ''), 0.239299685383505)
        self.assertEqual(self.cmp.dist('', 'a'), 0.239299685383505)
        self.assertEqual(self.cmp.dist('abc', ''), 0.2583083415411729)
        self.assertEqual(self.cmp.dist('', 'abc'), 0.2583083415411729)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist('abcd', 'efgh'), 0.5231483280982145)

        self.assertAlmostEqual(self.cmp.dist('Nigel', 'Niall'), 0.4255706162)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Nigel'), 0.4255706162)
        self.assertAlmostEqual(self.cmp.dist('Colin', 'Coiln'), 0.4255706162)
        self.assertAlmostEqual(self.cmp.dist('Coiln', 'Colin'), 0.4255706162)
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 0.4090971174
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.dist('', ''), 0.0)
        self.assertAlmostEqual(
            self.cmp_no_d.dist('a', ''), 0.048841293631356586
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('', 'a'), 0.048841293631356586
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('abc', ''), 0.06906597261157077
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('', 'abc'), 0.06906597261157077
        )
        self.assertEqual(self.cmp_no_d.dist('abc', 'abc'), 0.0)
        self.assertAlmostEqual(
            self.cmp_no_d.dist('abcd', 'efgh'), 0.5251846303074061
        )

        self.assertAlmostEqual(
            self.cmp_no_d.dist('Nigel', 'Niall'), 0.4783390621
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('Niall', 'Nigel'), 0.4783390621
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('Colin', 'Coiln'), 0.4783390621
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('Coiln', 'Colin'), 0.4783390621
        )
        self.assertAlmostEqual(
            self.cmp_no_d.dist('ATCAACGAGT', 'AACGATTAG'), 0.4467094163
        )

    def test_stiles_sim_score(self):
        """Test abydos.distance.Stiles.sim_score."""
        # Base cases
        self.assertAlmostEqual(self.cmp.sim_score('', ''), 16.292255897915638)
        self.assertAlmostEqual(self.cmp.sim_score('a', ''), 8.992335212208333)
        self.assertAlmostEqual(self.cmp.sim_score('', 'a'), 8.992335212208333)
        self.assertAlmostEqual(
            self.cmp.sim_score('abc', ''), 8.692417367356594
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('', 'abc'), 8.692417367356594
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('abc', 'abc'), 17.98245215160657
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('abcd', 'efgh'), -0.8426334527850912
        )

        self.assertAlmostEqual(
            self.cmp.sim_score('Nigel', 'Niall'), 2.7352860243
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Niall', 'Nigel'), 2.7352860243
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Colin', 'Coiln'), 2.7352860243
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Coiln', 'Colin'), 2.7352860243
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('ATCAACGAGT', 'AACGATTAG'), 3.4428002638
        )

        # Tests with alphabet=0 (no d factor)
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('', ''), 13.647817481888637
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('a', ''), 13.22184890168726
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('', 'a'), 13.22184890168726
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('abc', ''), 13.522878821349728
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('', 'abc'), 13.522878821349728
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('abc', 'abc'), 15.69019612345796
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('abcd', 'efgh'), -0.8061799153541304
        )

        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Nigel', 'Niall'), 0.7043650362
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Niall', 'Nigel'), 0.7043650362
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Colin', 'Coiln'), 0.7043650362
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Coiln', 'Colin'), 0.7043650362
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('ATCAACGAGT', 'AACGATTAG'), 1.8208082871
        )

    def test_stiles_corr(self):
        """Test abydos.distance.Stiles.corr."""
        # Base cases
        self.assertEqual(self.cmp.corr('', ''), 1.0)
        self.assertAlmostEqual(self.cmp.corr('a', ''), 0.5214006292329901)
        self.assertAlmostEqual(self.cmp.corr('', 'a'), 0.5214006292329901)
        self.assertAlmostEqual(self.cmp.corr('abc', ''), 0.48338331691765435)
        self.assertAlmostEqual(self.cmp.corr('', 'abc'), 0.48338331691765435)
        self.assertEqual(self.cmp.corr('abc', 'abc'), 1.0)
        self.assertAlmostEqual(
            self.cmp.corr('abcd', 'efgh'), -0.046296656196428934
        )

        self.assertAlmostEqual(self.cmp.corr('Nigel', 'Niall'), 0.1488587676)
        self.assertAlmostEqual(self.cmp.corr('Niall', 'Nigel'), 0.1488587676)
        self.assertAlmostEqual(self.cmp.corr('Colin', 'Coiln'), 0.1488587676)
        self.assertAlmostEqual(self.cmp.corr('Coiln', 'Colin'), 0.1488587676)
        self.assertAlmostEqual(
            self.cmp.corr('ATCAACGAGT', 'AACGATTAG'), 0.1818057652
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.corr('', ''), 1.0)
        self.assertAlmostEqual(self.cmp_no_d.corr('a', ''), 0.9023174127372868)
        self.assertAlmostEqual(self.cmp_no_d.corr('', 'a'), 0.9023174127372868)
        self.assertAlmostEqual(
            self.cmp_no_d.corr('abc', ''), 0.8618680547768583
        )
        self.assertAlmostEqual(
            self.cmp_no_d.corr('', 'abc'), 0.8618680547768583
        )
        self.assertEqual(self.cmp_no_d.corr('abc', 'abc'), 1.0)
        self.assertAlmostEqual(
            self.cmp_no_d.corr('abcd', 'efgh'), -0.05036926061481227
        )

        self.assertAlmostEqual(
            self.cmp_no_d.corr('Nigel', 'Niall'), 0.0433218759
        )
        self.assertAlmostEqual(
            self.cmp_no_d.corr('Niall', 'Nigel'), 0.0433218759
        )
        self.assertAlmostEqual(
            self.cmp_no_d.corr('Colin', 'Coiln'), 0.0433218759
        )
        self.assertAlmostEqual(
            self.cmp_no_d.corr('Coiln', 'Colin'), 0.0433218759
        )
        self.assertAlmostEqual(
            self.cmp_no_d.corr('ATCAACGAGT', 'AACGATTAG'), 0.1065811673
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_levenshtein.

This module contains unit tests for abydos.distance.Levenshtein
"""

import unittest

from abydos.distance import Levenshtein


class LevenshteinTestCases(unittest.TestCase):
    """Test Levenshtein functions.

    abydos.distance.Levenshtein
    """

    cmp = Levenshtein()
    cmp_taper = Levenshtein(taper=True)

    def test_levenshtein_dist_abs(self):
        """Test abydos.distance.Levenshtein.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0)

        # http://oldfashionedsoftware.com/tag/levenshtein-distance/
        self.assertEqual(self.cmp.dist_abs('a', ''), 1)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 3)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 3)
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('a', 'a'), 0)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('a', 'ab'), 1)
        self.assertEqual(self.cmp.dist_abs('b', 'ab'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'abc'), 1)
        self.assertEqual(self.cmp.dist_abs('abcdefg', 'xabxcdxxefxgx'), 6)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('xabxcdxxefxgx', 'abcdefg'), 6)
        self.assertEqual(self.cmp.dist_abs('a', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'bc'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'axc'), 1)
        self.assertEqual(
            self.cmp.dist_abs('xabxcdxxefxgx', '1ab2cd34ef5g6'), 6
        )
        self.assertEqual(self.cmp.dist_abs('example', 'samples'), 3)
        self.assertEqual(self.cmp.dist_abs('sturgeon', 'urgently'), 6)
        self.assertEqual(self.cmp.dist_abs('levenshtein', 'frankenstein'), 6)
        self.assertEqual(self.cmp.dist_abs('distance', 'difference'), 5)
        self.assertEqual(
            self.cmp.dist_abs('java was neat', 'scala is great'), 7
        )

        # https://en.wikipedia.org/wiki/Damerau%E2%80%93Levenshtein_distance
        self.assertEqual(Levenshtein(mode='osa').dist_abs('CA', 'ABC'), 3)

        # test cost of insert
        self.assertEqual(
            Levenshtein(mode='lev', cost=(5, 7, 10, 10)).dist_abs('', 'b'), 5
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(5, 7, 10, 10)).dist_abs('', 'b'), 5
        )
        self.assertEqual(
            Levenshtein(mode='lev', cost=(5, 7, 10, 10)).dist_abs('a', 'ab'), 5
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(5, 7, 10, 10)).dist_abs('a', 'ab'), 5
        )

        # test cost of delete
        self.assertEqual(
            Levenshtein(mode='lev', cost=(5, 7, 10, 10)).dist_abs('b', ''), 7
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(5, 7, 10, 10)).dist_abs('b', ''), 7
        )
        self.assertEqual(
            Levenshtein(mode='lev', cost=(5, 7, 10, 10)).dist_abs('ab', 'a'), 7
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(5, 7, 10, 10)).dist_abs('ab', 'a'), 7
        )

        # test cost of substitute
        self.assertEqual(
            Levenshtein(mode='lev', cost=(10, 10, 5, 10)).dist_abs('a', 'b'), 5
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(10, 10, 5, 10)).dist_abs('a', 'b'), 5
        )
        self.assertEqual(
            Levenshtein(mode='lev', cost=(10, 10, 5, 10)).dist_abs('ac', 'bc'),
            5,
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(10, 10, 5, 10)).dist_abs('ac', 'bc'),
            5,
        )

        # test cost of transpose
        self.assertEqual(
            Levenshtein(mode='lev', cost=(10, 10, 10, 5)).dist_abs('ab', 'ba'),
            20,
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(10, 10, 10, 5)).dist_abs('ab', 'ba'),
            5,
        )
        self.assertEqual(
            Levenshtein(mode='lev', cost=(10, 10, 10, 5)).dist_abs(
                'abc', 'bac'
            ),
            20,
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(10, 10, 10, 5)).dist_abs(
                'abc', 'bac'
            ),
            5,
        )
        self.assertEqual(
            Levenshtein(mode='lev', cost=(10, 10, 10, 5)).dist_abs(
                'cab', 'cba'
            ),
            20,
        )
        self.assertEqual(
            Levenshtein(mode='osa', cost=(10, 10, 10, 5)).dist_abs(
                'cab', 'cba'
            ),
            5,
        )

        # tapered variant
        self.assertAlmostEqual(
            self.cmp_taper.dist_abs('abc', 'ac'), 1.33333333333
        )
        self.assertAlmostEqual(
            self.cmp_taper.dist_abs('xabxcdxxefxgx', 'abcdefg'),
            8.615384615384617,
        )
        self.assertAlmostEqual(
            self.cmp_taper.dist_abs('levenshtein', 'frankenstein'), 10
        )
        self.assertAlmostEqual(
            self.cmp_taper.dist_abs('distance', 'difference'),
            7.499999999999999,
        )

    def test_levenshtein_dist(self):
        """Test abydos.distance.Levenshtein.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)

        self.assertEqual(self.cmp.dist('a', 'a'), 0)
        self.assertEqual(self.cmp.dist('ab', 'ab'), 0)
        self.assertEqual(self.cmp.dist('', 'a'), 1)
        self.assertEqual(self.cmp.dist('', 'ab'), 1)
        self.assertEqual(self.cmp.dist('a', 'c'), 1)

        self.assertAlmostEqual(self.cmp.dist('abc', 'ac'), 1 / 3)
        self.assertAlmostEqual(self.cmp.dist('abbc', 'ac'), 1 / 2)
        self.assertAlmostEqual(self.cmp.dist('abbc', 'abc'), 1 / 4)

        # tapered variant
        self.assertAlmostEqual(
            self.cmp_taper.dist('abc', 'ac'), 0.2666666666666666
        )
        self.assertAlmostEqual(
            self.cmp_taper.dist('abbc', 'ac'), 0.4230769230769231
        )
        self.assertAlmostEqual(
            self.cmp_taper.dist('abbc', 'abc'), 0.19230769230769232
        )

    def test_levenshtein_sim(self):
        """Test abydos.distance.Levenshtein.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)

        self.assertEqual(self.cmp.sim('a', 'a'), 1)
        self.assertEqual(self.cmp.sim('ab', 'ab'), 1)
        self.assertEqual(self.cmp.sim('', 'a'), 0)
        self.assertEqual(self.cmp.sim('', 'ab'), 0)
        self.assertEqual(self.cmp.sim('a', 'c'), 0)

        self.assertAlmostEqual(self.cmp.sim('abc', 'ac'), 2 / 3)
        self.assertAlmostEqual(self.cmp.sim('abbc', 'ac'), 1 / 2)
        self.assertAlmostEqual(self.cmp.sim('abbc', 'abc'), 3 / 4)

    def test_levenshtein_alignment(self):
        """Test abydos.distance.Levenshtein.alignment."""
        self.assertEqual(self.cmp.alignment('', ''), (0, '', ''))

        self.assertEqual(self.cmp.alignment('a', 'a'), (0.0, 'a', 'a'))
        self.assertEqual(self.cmp.alignment('ab', 'ab'), (0.0, 'ab', 'ab'))
        self.assertEqual(self.cmp.alignment('', 'a'), (1.0, '-', 'a'))
        self.assertEqual(self.cmp.alignment('', 'ab'), (2.0, '--', 'ab'))
        self.assertEqual(self.cmp.alignment('a', 'c'), (1.0, 'a', 'c'))

        self.assertEqual(self.cmp.alignment('abc', 'ac'), (1.0, 'abc', 'a-c'))
        self.assertEqual(
            self.cmp.alignment('abbc', 'ac'), (2.0, 'abbc', 'a--c')
        )
        self.assertEqual(
            self.cmp.alignment('abbc', 'abc'), (1.0, 'abbc', 'ab-c')
        )

        self.assertEqual(
            Levenshtein(mode='osa').alignment('Niall', 'Naill'),
            (1.0, 'Niall', 'Naill'),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_phonet.

This module contains unit tests for abydos.phonetic.Phonet
"""

import codecs
import unittest

from abydos.phonetic import Phonet

from .. import ALLOW_RANDOM, _corpus_file, _one_in


class PhonetTestCases(unittest.TestCase):
    """Test Phonet functions.

    test cases for abydos.phonetic.Phonet
    """

    pa = Phonet()
    pa_1 = Phonet(1)
    pa_2 = Phonet(2)
    pa_1none = Phonet(1, 'none')
    pa_2none = Phonet(2, 'none')

    def test_phonet_german(self):
        """Test abydos.phonetic.Phonet (German)."""
        self.assertEqual(self.pa.encode(''), '')

        # https://code.google.com/p/phonet4java/source/browse/trunk/src/test/java/com/googlecode/phonet4java/Phonet1Test.java
        self.assertEqual(self.pa_1.encode(''), '')
        self.assertEqual(self.pa_1.encode('Zedlitz'), 'ZETLIZ')
        self.assertEqual(self.pa_1.encode('Bremerhaven'), 'BREMAHAFN')
        self.assertEqual(self.pa_1.encode('Hamburger Hafen'), 'HAMBURGA HAFN')
        self.assertEqual(self.pa_1.encode('Jesper'), 'IESPA')
        self.assertEqual(self.pa_1.encode('elisabeth'), 'ELISABET')
        self.assertEqual(self.pa_1.encode('elisabet'), 'ELISABET')
        self.assertEqual(self.pa_1.encode('Ziegler'), 'ZIKLA')
        self.assertEqual(self.pa_1.encode('Scherer'), 'SHERA')
        self.assertEqual(self.pa_1.encode('Bartels'), 'BARTLS')
        self.assertEqual(self.pa_1.encode('Jansen'), 'IANSN')
        self.assertEqual(self.pa_1.encode('Sievers'), 'SIWAS')
        self.assertEqual(self.pa_1.encode('Michels'), 'MICHLS')
        self.assertEqual(self.pa_1.encode('Ewers'), 'EWERS')
        self.assertEqual(self.pa_1.encode('Evers'), 'EWERS')
        self.assertEqual(self.pa_1.encode('Wessels'), 'WESLS')
        self.assertEqual(self.pa_1.encode('Gottschalk'), 'GOSHALK')
        self.assertEqual(self.pa_1.encode('Brückmann'), 'BRÜKMAN')
        self.assertEqual(self.pa_1.encode('Blechschmidt'), 'BLECHSHMIT')
        self.assertEqual(self.pa_1.encode('Kolodziej'), 'KOLOTZI')
        self.assertEqual(self.pa_1.encode('Krauße'), 'KRAUSE')
        self.assertEqual(self.pa_1.encode('Cachel'), 'KESHL')

        self.assertEqual(self.pa_2.encode(''), '')
        self.assertEqual(self.pa_2.encode('Zedlitz'), 'ZETLIZ')
        self.assertEqual(self.pa_2.encode('Bremerhaven'), 'BRENAFN')
        self.assertEqual(self.pa_2.encode('Schönberg'), 'ZÖNBAK')
        self.assertEqual(self.pa_2.encode('Hamburger Hafen'), 'ANBURKA AFN')
        self.assertEqual(self.pa_2.encode('Ziegler'), 'ZIKLA')
        self.assertEqual(self.pa_2.encode('Scherer'), 'ZERA')
        self.assertEqual(self.pa_2.encode('Jansen'), 'IANZN')
        self.assertEqual(self.pa_2.encode('Eberhardt'), 'EBART')
        self.assertEqual(self.pa_2.encode('Gottschalk'), 'KUZALK')
        self.assertEqual(self.pa_2.encode('Brückmann'), 'BRIKNAN')
        self.assertEqual(self.pa_2.encode('Blechschmidt'), 'BLEKZNIT')
        self.assertEqual(self.pa_2.encode('Kolodziej'), 'KULUTZI')
        self.assertEqual(self.pa_2.encode('Krauße'), 'KRAUZE')

        # etc. (for code coverage)
        self.assertEqual(self.pa_1.encode('Jesper'), 'IESPA')
        self.assertEqual(self.pa_1.encode('Glacéhandschuh'), 'GLAZANSHU')
        self.assertEqual(self.pa_1.encode('Blechschmidt'), 'BLECHSHMIT')
        self.assertEqual(self.pa_1.encode('Burgdorf'), 'BURKDORF')
        self.assertEqual(self.pa_1.encode('Holzschuh'), 'HOLSHU')
        self.assertEqual(self.pa_1.encode('Aachen'), 'ACHN')
        self.assertEqual(
            self.pa_1.encode('Abendspaziergang'), 'ABENTSPAZIRGANK'
        )

    def test_phonet_nolang(self):
        """Test abydos.phonetic.Phonet (no language)."""
        self.assertEqual(Phonet(lang='none').encode(''), '')

        # https://code.google.com/p/phonet4java/source/browse/trunk/src/test/java/com/googlecode/phonet4java/Phonet1Test.java
        self.assertEqual(self.pa_1none.encode(''), '')
        self.assertEqual(self.pa_1none.encode('Zedlitz'), 'ZEDLITZ')
        self.assertEqual(self.pa_1none.encode('Bremerhaven'), 'BREMERHAVEN')
        self.assertEqual(self.pa_2none.encode('Schönberg'), 'SCHOENBERG')
        self.assertEqual(self.pa_1none.encode('Brückmann'), 'BRUECKMAN')
        self.assertEqual(self.pa_1none.encode('Krauße'), 'KRAUSE')

        self.assertEqual(self.pa_2none.encode(''), '')
        self.assertEqual(self.pa_2none.encode('Zedlitz'), 'ZEDLITZ')
        self.assertEqual(self.pa_2none.encode('Bremerhaven'), 'BREMERHAVEN')
        self.assertEqual(self.pa_2none.encode('Schönberg'), 'SCHOENBERG')
        self.assertEqual(self.pa_2none.encode('Brückmann'), 'BRUECKMAN')
        self.assertEqual(self.pa_2none.encode('Krauße'), 'KRAUSE')

    def test_phonet_nachnamen(self):
        """Test abydos.phonetic.Phonet (Nachnamen set)."""
        if not ALLOW_RANDOM:
            return
        with codecs.open(
            _corpus_file('nachnamen.csv'), encoding='utf-8'
        ) as nachnamen_testset:
            for nn_line in nachnamen_testset:
                if nn_line[0] != '#':
                    nn_line = nn_line.strip().split(',')
                    # This test set is very large (~10000 entries)
                    # so let's just randomly select about 100 for testing
                    if len(nn_line) >= 3 and _one_in(100):
                        (term, ph1, ph2) = nn_line
                        self.assertEqual(self.pa_1.encode(term), ph1)
                        self.assertEqual(self.pa_2.encode(term), ph2)

    def test_phonet_ngerman(self):
        """Test abydos.phonetic.Phonet (ngerman set)."""
        if not ALLOW_RANDOM:
            return
        with codecs.open(
            _corpus_file('ngerman.csv'), encoding='utf-8'
        ) as ngerman_testset:
            for ng_line in ngerman_testset:
                if ng_line[0] != '#':
                    ng_line = ng_line.strip().split(',')
                    # This test set is very large (~3000000 entries)
                    # so let's just randomly select about 30 for testing
                    if len(ng_line) >= 3 and _one_in(10000):
                        (term, ph1, ph2) = ng_line
                        self.assertEqual(self.pa_1.encode(term), ph1)
                        self.assertEqual(self.pa_2.encode(term), ph2)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_monge_elkan.

This module contains unit tests for abydos.distance.MongeElkan
"""

import unittest

from abydos.distance import Jaccard, MongeElkan


class MongeElkanTestCases(unittest.TestCase):
    """Test Monge-Elkan functions.

    abydos.distance.MongeElkan
    """

    cmp = MongeElkan()
    cmp_sym = MongeElkan(symmetric=True)
    cmp_jac = MongeElkan(sim_func=Jaccard())
    cmp_jac_sim = MongeElkan(sim_func=Jaccard().sim)

    def test_monge_elkan_sim(self):
        """Test abydos.distance.MongeElkan.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('', 'a'), 0)
        self.assertEqual(self.cmp.sim('a', 'a'), 1)

        self.assertEqual(self.cmp.sim('Niall', 'Neal'), 3 / 4)
        self.assertEqual(self.cmp.sim('Niall', 'Njall'), 5 / 6)
        self.assertEqual(self.cmp.sim('Niall', 'Niel'), 3 / 4)
        self.assertEqual(self.cmp.sim('Niall', 'Nigel'), 3 / 4)

        self.assertEqual(self.cmp_sym.sim('Niall', 'Neal'), 31 / 40)
        self.assertEqual(self.cmp_sym.sim('Niall', 'Njall'), 5 / 6)
        self.assertEqual(self.cmp_sym.sim('Niall', 'Niel'), 31 / 40)
        self.assertAlmostEqual(self.cmp_sym.sim('Niall', 'Nigel'), 17 / 24)

        self.assertEqual(self.cmp_jac.sim('Njall', 'Neil'), 29 / 60)
        self.assertEqual(self.cmp_jac_sim.sim('Njall', 'Neil'), 29 / 60)

    def test_monge_elkan_dist(self):
        """Test abydos.distance.MongeElkan.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('', 'a'), 1)

        self.assertEqual(self.cmp.dist('Niall', 'Neal'), 1 / 4)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Njall'), 1 / 6)
        self.assertEqual(self.cmp.dist('Niall', 'Niel'), 1 / 4)
        self.assertEqual(self.cmp.dist('Niall', 'Nigel'), 1 / 4)

        self.assertAlmostEqual(self.cmp_sym.dist('Niall', 'Neal'), 9 / 40)
        self.assertAlmostEqual(self.cmp_sym.dist('Niall', 'Njall'), 1 / 6)
        self.assertAlmostEqual(self.cmp_sym.dist('Niall', 'Niel'), 9 / 40)
        self.assertAlmostEqual(self.cmp_sym.dist('Niall', 'Nigel'), 7 / 24)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance__distance.

This module contains unit tests for abydos.distance._Distance
"""

import unittest

from abydos.distance import Dice, Levenshtein


class DistanceTestCases(unittest.TestCase):
    """Test _Distance base class.

    abydos.distance._Distance.sim, .dist, & .dist_abs
    """

    lev = Levenshtein()
    dice = Dice()

    def test_sim(self):
        """Test abydos.distance._Distance.sim."""
        self.assertEqual(
            self.lev.sim('Niall', 'Nigel'),
            1.0 - self.lev.dist('Niall', 'Nigel'),
        )
        self.assertEqual(
            self.dice.dist('Niall', 'Nigel'),
            1.0 - self.dice.sim('Niall', 'Nigel'),
        )

    def test_dist(self):
        """Test abydos.distance._Distance.dist."""
        self.assertEqual(
            1.0 - self.lev.sim('Niall', 'Nigel'),
            self.lev.dist('Niall', 'Nigel'),
        )
        self.assertEqual(
            1.0 - self.dice.dist('Niall', 'Nigel'),
            self.dice.sim('Niall', 'Nigel'),
        )

    def test_dist_abs(self):
        """Test abydos.distance._Distance.dist_abs."""
        self.assertEqual(
            self.dice.dist('Niall', 'Nigel'),
            self.dice.dist_abs('Niall', 'Nigel'),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stats.test_stats_pairwise.

This module contains unit tests for abydos.stats pairwise functions
"""

import unittest

from abydos.distance import Jaccard, JaroWinkler
from abydos.stats import (
    amean,
    gmean,
    hmean,
    mean_pairwise_similarity,
    pairwise_similarity_statistics,
)

NIALL = (
    'Niall',
    'Neal',
    'Neil',
    'Njall',
    'Njáll',
    'Nigel',
    'Neel',
    'Nele',
    'Nigelli',
    'Nel',
    'Kneale',
    'Uí Néill',
    "O'Neill",
    'MacNeil',
    'MacNele',
    'Niall Noígíallach',
)

NIALL_1WORD = (
    'Niall',
    'Neal',
    'Neil',
    'Njall',
    'Njáll',
    'Nigel',
    'Neel',
    'Nele',
    'Nigelli',
    'Nel',
    'Kneale',
    "O'Neill",
    'MacNeil',
    'MacNele',
)


class MPSTestCases(unittest.TestCase):
    """Test mean pairwise similarity functions.

    abydos.stats.mean_pairwise_similarity
    """

    def test_mean_pairwise_similarity(self):
        """Test abydos.stats.mean_pairwise_similarity."""
        self.assertEqual(mean_pairwise_similarity(NIALL), 0.29362587170180671)
        self.assertEqual(
            mean_pairwise_similarity(NIALL, symmetric=True), 0.2936258717018066
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=hmean),
            0.29362587170180671,
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=hmean, symmetric=True),
            0.2936258717018066,
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=gmean),
            0.33747245800668441,
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=gmean, symmetric=True),
            0.33747245800668441,
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=amean),
            0.38009278711484601,
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL, mean_func=amean, symmetric=True),
            0.38009278711484623,
        )

        self.assertEqual(
            mean_pairwise_similarity(NIALL_1WORD),
            mean_pairwise_similarity(' '.join(NIALL_1WORD)),
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL_1WORD, symmetric=True),
            mean_pairwise_similarity(' '.join(NIALL_1WORD), symmetric=True),
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL_1WORD, mean_func=gmean),
            mean_pairwise_similarity(' '.join(NIALL_1WORD), mean_func=gmean),
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL_1WORD, mean_func=amean),
            mean_pairwise_similarity(' '.join(NIALL_1WORD), mean_func=amean),
        )

        self.assertRaises(ValueError, mean_pairwise_similarity, ['a b c'])
        self.assertRaises(ValueError, mean_pairwise_similarity, 'abc')
        self.assertRaises(ValueError, mean_pairwise_similarity, 0)
        self.assertRaises(
            ValueError, mean_pairwise_similarity, NIALL, mean_func='imaginary'
        )
        self.assertRaises(
            ValueError, mean_pairwise_similarity, NIALL, metric='imaginary'
        )

        self.assertEqual(
            mean_pairwise_similarity(NIALL),
            mean_pairwise_similarity(tuple(NIALL)),
        )
        self.assertEqual(
            mean_pairwise_similarity(NIALL),
            mean_pairwise_similarity(list(NIALL)),
        )
        self.assertAlmostEqual(
            mean_pairwise_similarity(NIALL),
            mean_pairwise_similarity(sorted(NIALL)),
        )
        self.assertAlmostEqual(
            mean_pairwise_similarity(NIALL),
            mean_pairwise_similarity(set(NIALL)),
        )


class PSSTestCases(unittest.TestCase):
    """Test pairwise similarity statistics functions.

    abydos.stats.pairwise_similarity_statistics
    """

    def test_pairwise_similarity_statistics(self):
        """Test abydos.stats.pairwise_similarity_statistics."""
        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, NIALL
        )
        self.assertAlmostEqual(pw_max, 1.0)
        self.assertAlmostEqual(pw_min, 0.11764705882352944)
        self.assertAlmostEqual(pw_mean, 0.4188369879201684)
        self.assertAlmostEqual(pw_std, 0.2265099631340623)

        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, ('Kneal',)
        )
        self.assertAlmostEqual(pw_max, 0.8333333333333334)
        self.assertAlmostEqual(pw_min, 0.11764705882352944)
        self.assertAlmostEqual(pw_mean, 0.30474877450980387)
        self.assertAlmostEqual(pw_std, 0.1842666797571549)

        # Test symmetric
        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, NIALL, symmetric=True
        )
        self.assertAlmostEqual(pw_max, 1.0)
        self.assertAlmostEqual(pw_min, 0.11764705882352944)
        self.assertAlmostEqual(pw_mean, 0.4188369879201679)
        self.assertAlmostEqual(pw_std, 0.22650996313406255)

        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, ('Kneal',), symmetric=True
        )
        self.assertAlmostEqual(pw_max, 0.8333333333333334)
        self.assertAlmostEqual(pw_min, 0.11764705882352944)
        self.assertAlmostEqual(pw_mean, 0.304748774509804)
        self.assertAlmostEqual(pw_std, 0.18426667975715486)

        # Test with splittable strings
        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            'The quick brown fox', 'jumped over the lazy dog.'
        )
        self.assertAlmostEqual(pw_max, 0.6666666666666667)
        self.assertAlmostEqual(pw_min, 0.0)
        self.assertAlmostEqual(pw_mean, 0.08499999999999999)
        self.assertAlmostEqual(pw_std, 0.16132265804901677)

        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            'The', 'jumped'
        )
        self.assertAlmostEqual(pw_max, 0.16666666666666663)
        self.assertAlmostEqual(pw_min, 0.16666666666666663)
        self.assertAlmostEqual(pw_mean, 0.16666666666666663)
        self.assertAlmostEqual(pw_std, 0.0)

        # Test with a set metric
        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, NIALL, metric=Jaccard().sim
        )
        self.assertAlmostEqual(pw_max, 1.0)
        self.assertAlmostEqual(pw_min, 0.0)
        self.assertAlmostEqual(pw_mean, 0.23226906681010506)
        self.assertAlmostEqual(pw_std, 0.24747101181262784)

        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, NIALL, metric=JaroWinkler().dist
        )
        self.assertAlmostEqual(pw_max, 1.0)
        self.assertAlmostEqual(pw_min, 0.0)
        self.assertAlmostEqual(pw_mean, 0.3352660334967324)
        self.assertAlmostEqual(pw_std, 0.18394505847524578)

        # Test using hmean'
        (pw_max, pw_min, pw_mean, pw_std) = pairwise_similarity_statistics(
            NIALL, NIALL, mean_func=hmean
        )
        self.assertAlmostEqual(pw_max, 1.0)
        self.assertAlmostEqual(pw_min, 0.11764705882352944)
        self.assertAlmostEqual(pw_mean, 0.30718771249150056)
        self.assertAlmostEqual(pw_std, 0.25253182790044676)

        # Test exceptions
        self.assertRaises(
            ValueError,
            pairwise_similarity_statistics,
            NIALL,
            NIALL,
            mean_func='mean',
        )
        self.assertRaises(
            ValueError,
            pairwise_similarity_statistics,
            NIALL,
            NIALL,
            metric='Levenshtein',
        )
        self.assertRaises(ValueError, pairwise_similarity_statistics, 5, NIALL)
        self.assertRaises(ValueError, pairwise_similarity_statistics, NIALL, 5)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_clef_swedish.

This module contains unit tests for abydos.stemmer.CLEFSwedish
"""

import unittest

from abydos.stemmer import CLEFSwedish


class CLEFTestCases(unittest.TestCase):
    """Test CLEF Swedish functions.

    abydos.stemmer.CLEFSwedish
    """

    stmr = CLEFSwedish()

    def test_clef_swedish(self):
        """Test abydos.stemmer.CLEFSwedish."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        # unstemmed
        self.assertEqual(self.stmr.stem('konung'), 'konung')

        # len <= 3
        self.assertEqual(self.stmr.stem('km'), 'km')
        self.assertEqual(self.stmr.stem('ja'), 'ja')
        self.assertEqual(self.stmr.stem('de'), 'de')
        self.assertEqual(self.stmr.stem('in'), 'in')
        self.assertEqual(self.stmr.stem('a'), 'a')
        self.assertEqual(self.stmr.stem('mer'), 'mer')
        self.assertEqual(self.stmr.stem('s'), 's')
        self.assertEqual(self.stmr.stem('e'), 'e')
        self.assertEqual(self.stmr.stem('oss'), 'oss')
        self.assertEqual(self.stmr.stem('hos'), 'hos')

        # genitive
        self.assertEqual(self.stmr.stem('svenskars'), 'svensk')
        self.assertEqual(self.stmr.stem('stadens'), 'stad')
        self.assertEqual(self.stmr.stem('kommuns'), 'kommu')
        self.assertEqual(self.stmr.stem('aftonbladets'), 'aftonblad')

        # len > 7
        self.assertEqual(self.stmr.stem('fängelser'), 'fäng')
        self.assertEqual(self.stmr.stem('möjligheten'), 'möjlig')

        # len > 6
        self.assertEqual(self.stmr.stem('svenskar'), 'svensk')
        self.assertEqual(self.stmr.stem('myndigheterna'), 'myndighet')
        self.assertEqual(self.stmr.stem('avgörande'), 'avgör')
        self.assertEqual(self.stmr.stem('fängelse'), 'fäng')
        self.assertEqual(self.stmr.stem('viktigaste'), 'viktig')
        self.assertEqual(self.stmr.stem('kvinnorna'), 'kvinn')
        self.assertEqual(self.stmr.stem('åklagaren'), 'åklag')

        # len > 5
        self.assertEqual(self.stmr.stem('tidigare'), 'tidig')
        self.assertEqual(self.stmr.stem('senast'), 'sen')
        self.assertEqual(self.stmr.stem('möjlighet'), 'möjlig')

        # len > 4
        self.assertEqual(self.stmr.stem('svenskar'), 'svensk')
        self.assertEqual(self.stmr.stem('skriver'), 'skriv')
        self.assertEqual(self.stmr.stem('människor'), 'människ')
        self.assertEqual(self.stmr.stem('staden'), 'stad')
        self.assertEqual(self.stmr.stem('kunnat'), 'kunn')
        self.assertEqual(self.stmr.stem('samarbete'), 'samarbe')
        self.assertEqual(self.stmr.stem('aftonbladet'), 'aftonblad')

        # len > 3
        self.assertEqual(self.stmr.stem('allt'), 'all')
        self.assertEqual(self.stmr.stem('vilka'), 'vilk')
        self.assertEqual(self.stmr.stem('länge'), 'läng')
        self.assertEqual(self.stmr.stem('kommun'), 'kommu')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_generalized_fleiss.

This module contains unit tests for abydos.distance.GeneralizedFleiss
"""

import unittest

from abydos.distance import GeneralizedFleiss


class GeneralizedFleissTestCases(unittest.TestCase):
    """Test GeneralizedFleiss functions.

    abydos.distance.GeneralizedFleiss
    """

    cmp = GeneralizedFleiss(marginals='a')
    cmp_no_d = GeneralizedFleiss(alphabet=0)
    cmp_b = GeneralizedFleiss(marginals='b')
    cmp_c = GeneralizedFleiss(marginals='c')
    cmp_prop = GeneralizedFleiss(proportional=True)
    cmp_quad = GeneralizedFleiss(mean_func='quadratic')
    cmp_hero = GeneralizedFleiss(mean_func='heronian')
    cmp_ag = GeneralizedFleiss(mean_func='ag')
    cmp_gh = GeneralizedFleiss(mean_func='gh')
    cmp_agh = GeneralizedFleiss(mean_func='agh')

    def test_generalized_fleiss_sim(self):
        """Test abydos.distance.GeneralizedFleiss.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 0.5)
        self.assertEqual(self.cmp.sim('a', ''), 0.5)
        self.assertEqual(self.cmp.sim('', 'a'), 0.5)
        self.assertEqual(self.cmp.sim('abc', ''), 0.5)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.5)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(self.cmp.sim('Nigel', 'Niall'), 0.7480719794)
        self.assertAlmostEqual(self.cmp.sim('Niall', 'Nigel'), 0.7480719794)
        self.assertAlmostEqual(self.cmp.sim('Colin', 'Coiln'), 0.7480719794)
        self.assertAlmostEqual(self.cmp.sim('Coiln', 'Colin'), 0.7480719794)
        self.assertAlmostEqual(
            self.cmp.sim('ATCAACGAGT', 'AACGATTAG'), 0.8310964723
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.sim('', ''), 0.5)
        self.assertEqual(self.cmp_no_d.sim('a', ''), 0.5)
        self.assertEqual(self.cmp_no_d.sim('', 'a'), 0.5)
        self.assertEqual(self.cmp_no_d.sim('abc', ''), 0.5)
        self.assertEqual(self.cmp_no_d.sim('', 'abc'), 0.5)
        self.assertEqual(self.cmp_no_d.sim('abc', 'abc'), 0.5)
        self.assertEqual(self.cmp_no_d.sim('abcd', 'efgh'), 0.0)

        self.assertAlmostEqual(self.cmp_no_d.sim('Nigel', 'Niall'), 0.25)
        self.assertAlmostEqual(self.cmp_no_d.sim('Niall', 'Nigel'), 0.25)
        self.assertAlmostEqual(self.cmp_no_d.sim('Colin', 'Coiln'), 0.25)
        self.assertAlmostEqual(self.cmp_no_d.sim('Coiln', 'Colin'), 0.25)
        self.assertAlmostEqual(
            self.cmp_no_d.sim('ATCAACGAGT', 'AACGATTAG'), 0.3356164384
        )

        # marginals b
        self.assertEqual(self.cmp_b.sim('abc', 'abc'), 0.5051280702677116)
        self.assertEqual(self.cmp_b.sim('abcd', 'efgh'), 0.4999588047443752)

        self.assertAlmostEqual(
            self.cmp_b.sim('Nigel', 'Niall'), 0.5038260754642173
        )
        self.assertAlmostEqual(
            self.cmp_b.sim('Niall', 'Nigel'), 0.5038260754642173
        )
        self.assertAlmostEqual(
            self.cmp_b.sim('Colin', 'Coiln'), 0.5038260754642173
        )
        self.assertAlmostEqual(
            self.cmp_b.sim('Coiln', 'Colin'), 0.5038260754642173
        )
        self.assertAlmostEqual(
            self.cmp_b.sim('ATCAACGAGT', 'AACGATTAG'), 0.5089871192422611
        )

        # marginals c
        self.assertEqual(self.cmp_c.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_c.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(
            self.cmp_c.sim('Nigel', 'Niall'), 0.7480719794344473
        )
        self.assertAlmostEqual(
            self.cmp_c.sim('Niall', 'Nigel'), 0.7480719794344473
        )
        self.assertAlmostEqual(
            self.cmp_c.sim('Colin', 'Coiln'), 0.7480719794344473
        )
        self.assertAlmostEqual(
            self.cmp_c.sim('Coiln', 'Colin'), 0.7480719794344473
        )
        self.assertAlmostEqual(
            self.cmp_c.sim('ATCAACGAGT', 'AACGATTAG'), 0.8310760896330953
        )

        # proportional
        self.assertEqual(self.cmp_prop.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_prop.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(
            self.cmp_prop.sim('Nigel', 'Niall'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_prop.sim('Niall', 'Nigel'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_prop.sim('Colin', 'Coiln'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_prop.sim('Coiln', 'Colin'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_prop.sim('ATCAACGAGT', 'AACGATTAG'), 0.8310964723
        )

        # quadratic mean
        self.assertEqual(self.cmp_quad.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_quad.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(
            self.cmp_quad.sim('Nigel', 'Niall'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_quad.sim('Niall', 'Nigel'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_quad.sim('Colin', 'Coiln'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_quad.sim('Coiln', 'Colin'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_quad.sim('ATCAACGAGT', 'AACGATTAG'), 0.8307317829209393
        )

        # heronian mean
        self.assertEqual(self.cmp_hero.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_hero.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(
            self.cmp_hero.sim('Nigel', 'Niall'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_hero.sim('Niall', 'Nigel'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_hero.sim('Colin', 'Coiln'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_hero.sim('Coiln', 'Colin'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_hero.sim('ATCAACGAGT', 'AACGATTAG'), 0.8312183486929234
        )

        # ag mean
        self.assertEqual(self.cmp_ag.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_ag.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(self.cmp_ag.sim('Nigel', 'Niall'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_ag.sim('Niall', 'Nigel'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_ag.sim('Colin', 'Coiln'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_ag.sim('Coiln', 'Colin'), 0.7480719794)
        self.assertAlmostEqual(
            self.cmp_ag.sim('ATCAACGAGT', 'AACGATTAG'), 0.8312793457877081
        )

        # gh mean
        self.assertEqual(self.cmp_gh.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_gh.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(self.cmp_gh.sim('Nigel', 'Niall'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_gh.sim('Niall', 'Nigel'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_gh.sim('Colin', 'Coiln'), 0.7480719794)
        self.assertAlmostEqual(self.cmp_gh.sim('Coiln', 'Colin'), 0.7480719794)
        self.assertAlmostEqual(
            self.cmp_gh.sim('ATCAACGAGT', 'AACGATTAG'), 0.8316454969290444
        )

        # agh mean
        self.assertEqual(self.cmp_agh.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp_agh.sim('abcd', 'efgh'), 0.496790757381258)

        self.assertAlmostEqual(
            self.cmp_agh.sim('Nigel', 'Niall'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_agh.sim('Niall', 'Nigel'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_agh.sim('Colin', 'Coiln'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_agh.sim('Coiln', 'Colin'), 0.7480719794
        )
        self.assertAlmostEqual(
            self.cmp_agh.sim('ATCAACGAGT', 'AACGATTAG'), 0.8314623707995847
        )

    def test_generalized_fleiss_dist(self):
        """Test abydos.distance.GeneralizedFleiss.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0.5)
        self.assertEqual(self.cmp.dist('a', ''), 0.5)
        self.assertEqual(self.cmp.dist('', 'a'), 0.5)
        self.assertEqual(self.cmp.dist('abc', ''), 0.5)
        self.assertEqual(self.cmp.dist('', 'abc'), 0.5)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist('abcd', 'efgh'), 0.503209242618742)

        self.assertAlmostEqual(self.cmp.dist('Nigel', 'Niall'), 0.2519280206)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Nigel'), 0.2519280206)
        self.assertAlmostEqual(self.cmp.dist('Colin', 'Coiln'), 0.2519280206)
        self.assertAlmostEqual(self.cmp.dist('Coiln', 'Colin'), 0.2519280206)
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 0.1689035277
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.dist('', ''), 0.5)
        self.assertEqual(self.cmp_no_d.dist('a', ''), 0.5)
        self.assertEqual(self.cmp_no_d.dist('', 'a'), 0.5)
        self.assertEqual(self.cmp_no_d.dist('abc', ''), 0.5)
        self.assertEqual(self.cmp_no_d.dist('', 'abc'), 0.5)
        self.assertEqual(self.cmp_no_d.dist('abc', 'abc'), 0.5)
        self.assertEqual(self.cmp_no_d.dist('abcd', 'efgh'), 1.0)

        self.assertAlmostEqual(self.cmp_no_d.dist('Nigel', 'Niall'), 0.75)
        self.assertAlmostEqual(self.cmp_no_d.dist('Niall', 'Nigel'), 0.75)
        self.assertAlmostEqual(self.cmp_no_d.dist('Colin', 'Coiln'), 0.75)
        self.assertAlmostEqual(self.cmp_no_d.dist('Coiln', 'Colin'), 0.75)
        self.assertAlmostEqual(
            self.cmp_no_d.dist('ATCAACGAGT', 'AACGATTAG'), 0.6643835616
        )

    def test_generalized_fleiss_corr(self):
        """Test abydos.distance.GeneralizedFleiss.corr."""
        # Base cases
        self.assertEqual(self.cmp.corr('', ''), 0.0)
        self.assertEqual(self.cmp.corr('a', ''), 0.0)
        self.assertEqual(self.cmp.corr('', 'a'), 0.0)
        self.assertEqual(self.cmp.corr('abc', ''), 0.0)
        self.assertEqual(self.cmp.corr('', 'abc'), 0.0)
        self.assertEqual(self.cmp.corr('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.corr('abcd', 'efgh'), -0.006418485237483954)

        self.assertAlmostEqual(self.cmp.corr('Nigel', 'Niall'), 0.4961439589)
        self.assertAlmostEqual(self.cmp.corr('Niall', 'Nigel'), 0.4961439589)
        self.assertAlmostEqual(self.cmp.corr('Colin', 'Coiln'), 0.4961439589)
        self.assertAlmostEqual(self.cmp.corr('Coiln', 'Colin'), 0.4961439589)
        self.assertAlmostEqual(
            self.cmp.corr('ATCAACGAGT', 'AACGATTAG'), 0.6621929447
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.corr('', ''), 0.0)
        self.assertEqual(self.cmp_no_d.corr('a', ''), 0.0)
        self.assertEqual(self.cmp_no_d.corr('', 'a'), 0.0)
        self.assertEqual(self.cmp_no_d.corr('abc', ''), 0.0)
        self.assertEqual(self.cmp_no_d.corr('', 'abc'), 0.0)
        self.assertEqual(self.cmp_no_d.corr('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp_no_d.corr('abcd', 'efgh'), -1.0)

        self.assertAlmostEqual(self.cmp_no_d.corr('Nigel', 'Niall'), -0.5)
        self.assertAlmostEqual(self.cmp_no_d.corr('Niall', 'Nigel'), -0.5)
        self.assertAlmostEqual(self.cmp_no_d.corr('Colin', 'Coiln'), -0.5)
        self.assertAlmostEqual(self.cmp_no_d.corr('Coiln', 'Colin'), -0.5)
        self.assertAlmostEqual(
            self.cmp_no_d.corr('ATCAACGAGT', 'AACGATTAG'), -0.3287671233
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_norphone.

This module contains unit tests for abydos.phonetic.Norphone
"""

import unittest

from abydos.phonetic import Norphone


class NorphoneTestCases(unittest.TestCase):
    """Test Norphone functions.

    test cases for abydos.phonetic.Norphone
    """

    pa = Norphone()

    def test_norphone(self):
        """Test abydos.phonetic.Norphone."""
        # Base case
        self.assertEqual(self.pa.encode(''), '')

        # Examples given at
        # https://github.com/larsga/Duke/blob/master/duke-core/src/test/java/no/priv/garshol/duke/comparators/NorphoneComparatorTest.java
        self.assertEqual(
            self.pa.encode('Aarestad'), self.pa.encode('\u00C5rrestad')
        )
        self.assertEqual(
            self.pa.encode('Andreasen'), self.pa.encode('Andreassen')
        )
        self.assertEqual(self.pa.encode('Arntsen'), self.pa.encode('Arntzen'))
        self.assertEqual(self.pa.encode('Bache'), self.pa.encode('Bakke'))
        self.assertEqual(self.pa.encode('Frank'), self.pa.encode('Franck'))
        self.assertEqual(
            self.pa.encode('Christian'), self.pa.encode('Kristian')
        )
        self.assertEqual(
            self.pa.encode('Kielland'), self.pa.encode('Kjelland')
        )
        self.assertEqual(self.pa.encode('Krogh'), self.pa.encode('Krog'))
        self.assertEqual(self.pa.encode('Krog'), self.pa.encode('Krohg'))
        self.assertEqual(self.pa.encode('Jendal'), self.pa.encode('Jendahl'))
        self.assertEqual(self.pa.encode('Jendal'), self.pa.encode('Hjendal'))
        self.assertEqual(self.pa.encode('Jendal'), self.pa.encode('Gjendal'))
        self.assertEqual(self.pa.encode('Vold'), self.pa.encode('Wold'))
        self.assertEqual(self.pa.encode('Thomas'), self.pa.encode('Tomas'))
        self.assertEqual(self.pa.encode('Aamodt'), self.pa.encode('Aamot'))
        self.assertEqual(self.pa.encode('Aksel'), self.pa.encode('Axel'))
        self.assertEqual(
            self.pa.encode('Kristoffersen'), self.pa.encode('Christophersen')
        )
        self.assertEqual(self.pa.encode('Voll'), self.pa.encode('Vold'))
        self.assertEqual(self.pa.encode('Granli'), self.pa.encode('Granlid'))
        self.assertEqual(self.pa.encode('Gjever'), self.pa.encode('Giever'))
        self.assertEqual(
            self.pa.encode('Sannerhaugen'), self.pa.encode('Sanderhaugen')
        )
        self.assertEqual(self.pa.encode('Jahren'), self.pa.encode('Jaren'))
        self.assertEqual(
            self.pa.encode('Amundsrud'), self.pa.encode('Amundsr\u00F8d')
        )
        self.assertEqual(self.pa.encode('Karlson'), self.pa.encode('Carlson'))

        # Additional tests to increase coverage
        self.assertEqual(self.pa.encode('Århus'), 'ÅRHS')
        self.assertEqual(self.pa.encode('Skyrim'), 'XRM')
        self.assertEqual(self.pa.encode('kyss'), 'XS')
        self.assertEqual(self.pa.encode('Äthelwulf'), 'ÆTLVLF')
        self.assertEqual(self.pa.encode('eit'), 'ÆT')
        self.assertEqual(self.pa.encode('Öl'), 'ØL')

        # test cases by larsga (the algorithm's author) posted to Reddit
        # https://www.reddit.com/r/norge/comments/vksb5/norphone_mitt_forslag_til_en_norsk_soundex_vel/
        # modified, where necessary to match the "not implemented" rules
        # and rule added after the Reddit post
        reddit_tests = (
            (
                'MKLSN',
                (
                    'MICHALSEN',
                    'MIKKELSEN',
                    'MIKALSEN',
                    'MICHAELSEN',
                    'MIKAELSEN',
                    'MICKAELSEN',
                    'MICHELSEN',
                    'MIKELSEN',
                ),
            ),
            (
                'BRKR',
                (
                    'BERGER',
                    'BORGERUD',
                    'BURGER',
                    'BORGER',
                    'BORGAR',
                    'BIRGER',
                    'BRAGER',
                    'BERGERUD',
                ),
            ),
            (
                'TMS',
                (
                    'TOMMAS',
                    'THOMAS',
                    'THAMS',
                    'TOUMAS',
                    'THOMMAS',
                    'TIMMS',
                    'TOMAS',
                    'TUOMAS',
                ),
            ),
            (
                'HLR',
                (
                    'HOLER',
                    'HELLERUD',
                    'HALLRE',
                    'HOLLERUD',
                    'HILLER',
                    'HALLERUD',
                    'HOLLER',
                    'HALLER',
                ),
            ),
            (
                'MS',
                (
                    'MASS',
                    'MMS',
                    'MSS',
                    'MOES',
                    'MEZZO',
                    'MESA',
                    'MESSE',
                    'MOSS',
                ),
            ),
            (
                'HRST',
                (
                    'HIRSTI',
                    'HAARSETH',
                    'HAARSTAD',
                    'HARSTAD',
                    'HARESTUA',
                    'HERSETH',
                    'HERSTAD',
                    'HERSTUA',
                ),
            ),
            (
                'SVN',
                (
                    'SWANN',
                    'SVENI',
                    'SWAN',
                    'SVEN',
                    'SVEIN',
                    'SVEEN',
                    'SVENN',
                    'SVANE',
                ),
            ),
            (
                'SLT',
                (
                    'SELTE',
                    'SALT',
                    'SALTE',
                    'SLOTT',
                    'SLAATTO',
                    'SLETT',
                    'SLETTA',
                    'SLETTE',
                ),
            ),
            (
                'JNSN',
                (
                    'JANSSEN',
                    'JANSEN',
                    'JENSEN',
                    'JONASSEN',
                    'JANSON',
                    'JONSON',
                    'JENSSEN',
                    'JONSSON',
                ),
            ),
            (
                'ANRSN',
                (
                    'ANDRESSEN',
                    'ANDERSSON',
                    'ANDRESEN',
                    'ANDREASSEN',
                    'ANDERSEN',
                    'ANDERSON',
                    'ANDORSEN',
                    'ANDERSSEN',
                ),
            ),
            (
                'BRK',
                (
                    'BREKKE',
                    'BORCH',
                    'BRAKKE',
                    'BORK',
                    'BRECKE',
                    'BROCH',
                    'BRICK',
                    'BRUK',
                ),
            ),
            (
                'LN',
                (
                    'LINDE',
                    'LENDE',
                    'LUND',
                    'LAND',
                    'LINDA',
                    'LANDE',
                    'LIND',
                    'LUNDE',
                ),
            ),
            (
                'SF',
                (
                    'SOPHIE',
                    'SFE',
                    'SEFF',
                    'SEAFOOD',
                    'SOFIE',
                    'SAFE',
                    'SOFI',
                    'SOPHIA',
                ),
            ),
            (
                'BRST',
                (
                    'BRUASET',
                    'BUERSTAD',
                    'BARSTAD',
                    'BAARSTAD',
                    'BRUSETH',
                    'BERSTAD',
                    'BORSTAD',
                    'BRUSTAD',
                ),
            ),
            (
                'OLSN',
                (
                    'OHLSSON',
                    'OLESEN',
                    'OLSSON',
                    'OLAUSSON',
                    'OLAUSEN',
                    'OLAUSSEN',
                    'OLSEN',
                    'OLSON',
                ),
            ),
            (
                'MKL',
                (
                    'MIKAEL',
                    'MICHELA',
                    'MEIKLE',
                    'MIKAL',
                    'MIKKEL',
                    'MICHEL',
                    'MICHAL',
                    'MICHAEL',
                ),
            ),
            (
                'HR',
                (
                    'HEIER',
                    'HAR',
                    'HEER',
                    'HARRY',
                    'HEIR',
                    'HURRE',
                    'HERO',
                    'HUURRE',
                ),
            ),
            (
                'VLM',
                (
                    'VILLUM',
                    'WOLLUM',
                    'WILLIAM',
                    'WILLAM',
                    'WALLEM',
                    'WILLUM',
                    'VALUM',
                    'WILMO',
                ),
            ),
            (
                'SNS',
                (
                    'SYNNES',
                    'SINUS',
                    'SNUS',
                    'SNEIS',
                    'SANNES',
                    'SUNAAS',
                    'SUNNAAS',
                    'SAINES',
                ),
            ),
            (
                'SNL',
                (
                    'SANDAL',
                    'SANDAHL',
                    'SUNDEL',
                    'SANDLI',
                    'SUNNDAL',
                    'SANDELL',
                    'SANDLIE',
                    'SUNDAL',
                ),
            ),
            (
                'VK',
                ('VEKA', 'VIKA', 'WIIK', 'WOK', 'WIKE', 'WEEK', 'VIK', 'VIAK'),
            ),
            (
                'MTS',
                (
                    'METSO',
                    'MOTHES',
                    'MATHIAS',
                    'MATHIS',
                    'MATTIS',
                    'MYTHES',
                    'METOS',
                    'MATS',
                ),
            ),
        )
        for encoded, names in reddit_tests:
            for name in names:
                self.assertEqual(encoded, self.pa.encode(name))


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_dunning.

This module contains unit tests for abydos.distance.Dunning
"""

import unittest

from abydos.distance import Dunning


class DunningTestCases(unittest.TestCase):
    """Test Dunning functions.

    abydos.distance.Dunning
    """

    cmp = Dunning()
    cmp_no_d = Dunning(alphabet=0)

    def test_dunning_sim(self):
        """Test abydos.distance.Dunning.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertAlmostEqual(
            self.cmp.sim('abcd', 'efgh'), 0.0010606026735052122
        )

        self.assertAlmostEqual(self.cmp.sim('Nigel', 'Niall'), 0.3233318396)
        self.assertAlmostEqual(self.cmp.sim('Niall', 'Nigel'), 0.3233318396)
        self.assertAlmostEqual(self.cmp.sim('Colin', 'Coiln'), 0.3233318396)
        self.assertAlmostEqual(self.cmp.sim('Coiln', 'Colin'), 0.3233318396)
        self.assertAlmostEqual(
            self.cmp.sim('ATCAACGAGT', 'AACGATTAG'), 0.46141433614
        )

    def test_dunning_dist(self):
        """Test abydos.distance.Dunning.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0.0)
        self.assertEqual(self.cmp.dist('a', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'a'), 1.0)
        self.assertEqual(self.cmp.dist('abc', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'abc'), 1.0)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0.0)
        self.assertAlmostEqual(
            self.cmp.dist('abcd', 'efgh'), 0.9989393973264948
        )

        self.assertAlmostEqual(self.cmp.dist('Nigel', 'Niall'), 0.6766681604)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Nigel'), 0.6766681604)
        self.assertAlmostEqual(self.cmp.dist('Colin', 'Coiln'), 0.6766681604)
        self.assertAlmostEqual(self.cmp.dist('Coiln', 'Colin'), 0.6766681604)
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 0.53858566385
        )

    def test_dunning_sim_score(self):
        """Test abydos.distance.Dunning.sim_score."""
        # Base cases
        self.assertEqual(self.cmp.sim_score('', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('a', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'abc'), 0.0)
        self.assertAlmostEqual(self.cmp.sim_score('abc', 'abc'), 0.0923848802)
        self.assertAlmostEqual(
            self.cmp.sim_score('abcd', 'efgh'), 0.0001181119
        )

        self.assertAlmostEqual(
            self.cmp.sim_score('Nigel', 'Niall'), 0.0419023599
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Niall', 'Nigel'), 0.0419023599
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Colin', 'Coiln'), 0.0419023599
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Coiln', 'Colin'), 0.0419023599
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('ATCAACGAGT', 'AACGATTAG'), 0.098245766
        )

        # Tests with alphabet=0 (no d factor)
        self.assertEqual(self.cmp_no_d.sim_score('', ''), 0.0)
        self.assertEqual(self.cmp_no_d.sim_score('a', ''), 0.0)
        self.assertEqual(self.cmp_no_d.sim_score('', 'a'), 0.0)
        self.assertEqual(self.cmp_no_d.sim_score('abc', ''), 0.0)
        self.assertEqual(self.cmp_no_d.sim_score('', 'abc'), 0.0)
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('abc', 'abc'), 1.44385618977
        )
        self.assertAlmostEqual(self.cmp_no_d.sim_score('abcd', 'efgh'), 2.0)

        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Nigel', 'Niall'), 0.5032583348
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Niall', 'Nigel'), 0.5032583348
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Colin', 'Coiln'), 0.5032583348
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('Coiln', 'Colin'), 0.5032583348
        )
        self.assertAlmostEqual(
            self.cmp_no_d.sim_score('ATCAACGAGT', 'AACGATTAG'), 0.240203516
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._length.

Length similarity & distance
"""

from ._distance import _Distance

__all__ = ['Length']


class Length(_Distance):
    """Length similarity and distance.

    .. versionadded:: 0.3.6
    """

    def sim(self, src: str, tar: str) -> float:
        """Return the length similarity of two strings.

        Length similarity is the ratio of the length of the shorter string to
        the longer.

        Parameters
        ----------
        src : str
            Source string for comparison
        tar : str
            Target string for comparison

        Returns
        -------
        float
            Length similarity

        Examples
        --------
        >>> cmp = Length()
        >>> cmp.sim('cat', 'hat')
        1.0
        >>> cmp.sim('Niall', 'Neil')
        0.8
        >>> cmp.sim('aluminum', 'Catalan')
        0.875
        >>> cmp.sim('ATCG', 'TAGC')
        1.0


        .. versionadded:: 0.1.0
        .. versionchanged:: 0.3.6
            Encapsulated in class

        """
        if src == tar:
            return 1.0
        if not src or not tar:
            return 0.0
        return (
            len(src) / len(tar) if len(src) < len(tar) else len(tar) / len(src)
        )


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_mra.

This module contains unit tests for abydos.phonetic.MRA
"""

import unittest

from abydos.phonetic import MRA


class MraTestCases(unittest.TestCase):
    """Test MRA functions.

    test cases for abydos.phonetic.MRA
    """

    pa = MRA()

    def test_mra(self):
        """Test abydos.phonetic.MRA."""
        self.assertEqual(self.pa.encode(''), '')

        # https://en.wikipedia.org/wiki/Match_rating_approach
        self.assertEqual(self.pa.encode('Byrne'), 'BYRN')
        self.assertEqual(self.pa.encode('Boern'), 'BRN')
        self.assertEqual(self.pa.encode('Smith'), 'SMTH')
        self.assertEqual(self.pa.encode('Smyth'), 'SMYTH')
        self.assertEqual(self.pa.encode('Catherine'), 'CTHRN')
        self.assertEqual(self.pa.encode('Kathryn'), 'KTHRYN')

        # length checks
        self.assertEqual(self.pa.encode('Christopher'), 'CHRPHR')
        self.assertEqual(self.pa.encode('Dickensianistic'), 'DCKSTC')
        self.assertEqual(self.pa.encode('Acetylcholinesterase'), 'ACTTRS')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_phonetic_spanish.

This module contains unit tests for abydos.phonetic.PhoneticSpanish
"""

import unittest

from abydos.phonetic import PhoneticSpanish


class PhoneticSpanishTestCases(unittest.TestCase):
    """Test PhoneticSpanish functions.

    test cases for abydos.phonetic.PhoneticSpanish
    """

    pa = PhoneticSpanish()

    def test_phonetic_spanish(self):
        """Test abydos.phonetic.PhoneticSpanish."""
        # Base case
        self.assertEqual(self.pa.encode(''), '')

        # Examples given in
        self.assertEqual(self.pa.encode('Giraldo'), '8953')
        self.assertEqual(self.pa.encode('Jiraldo'), '8953')
        self.assertEqual(self.pa.encode('Halla'), '25')
        self.assertEqual(self.pa.encode('Haya'), '25')
        self.assertEqual(self.pa.encode('Cielo'), '45')
        self.assertEqual(self.pa.encode('Sielo'), '45')

        # Test to maximize coverage
        self.assertEqual(PhoneticSpanish(max_length=2).encode('Giraldo'), '89')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Giraldo'), 'GRLT')
        self.assertEqual(self.pa.encode_alpha('Jiraldo'), 'GRLT')
        self.assertEqual(self.pa.encode_alpha('Halla'), 'FL')
        self.assertEqual(self.pa.encode_alpha('Haya'), 'FL')
        self.assertEqual(self.pa.encode_alpha('Cielo'), 'SL')
        self.assertEqual(self.pa.encode_alpha('Sielo'), 'SL')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_mra.

This module contains unit tests for abydos.distance.MRA
"""

import unittest

from abydos.distance import MRA


class MraTestCases(unittest.TestCase):
    """Test MRA functions.

    abydos.distance.MRA
    """

    cmp = MRA()

    def test_mra_dist_abs(self):
        """Test abydos.distance.MRA.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 6)
        self.assertEqual(self.cmp.dist_abs('a', 'a'), 6)
        self.assertEqual(self.cmp.dist_abs('abcdefg', 'abcdefg'), 6)
        self.assertEqual(self.cmp.dist_abs('abcdefg', ''), 0)
        self.assertEqual(self.cmp.dist_abs('', 'abcdefg'), 0)

        # https://en.wikipedia.org/wiki/Match_rating_approach
        self.assertEqual(self.cmp.dist_abs('Byrne', 'Boern'), 5)
        self.assertEqual(self.cmp.dist_abs('Smith', 'Smyth'), 5)
        self.assertEqual(self.cmp.dist_abs('Catherine', 'Kathryn'), 4)

        self.assertEqual(self.cmp.dist_abs('ab', 'abcdefgh'), 0)
        self.assertEqual(self.cmp.dist_abs('ab', 'ac'), 5)
        self.assertEqual(self.cmp.dist_abs('abcdefik', 'abcdefgh'), 3)
        self.assertEqual(self.cmp.dist_abs('xyz', 'abc'), 0)

    def test_mra_sim(self):
        """Test abydos.distance.MRA.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('a', 'a'), 1)
        self.assertEqual(self.cmp.sim('abcdefg', 'abcdefg'), 1)
        self.assertEqual(self.cmp.sim('abcdefg', ''), 0)
        self.assertEqual(self.cmp.sim('', 'abcdefg'), 0)

        # https://en.wikipedia.org/wiki/Match_rating_approach
        self.assertEqual(self.cmp.sim('Byrne', 'Boern'), 5 / 6)
        self.assertEqual(self.cmp.sim('Smith', 'Smyth'), 5 / 6)
        self.assertEqual(self.cmp.sim('Catherine', 'Kathryn'), 4 / 6)

        self.assertEqual(self.cmp.sim('ab', 'abcdefgh'), 0)
        self.assertEqual(self.cmp.sim('ab', 'ac'), 5 / 6)
        self.assertEqual(self.cmp.sim('abcdefik', 'abcdefgh'), 3 / 6)
        self.assertEqual(self.cmp.sim('xyz', 'abc'), 0)

    def test_mra_dist(self):
        """Test abydos.distance.MRA.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('a', 'a'), 0)
        self.assertEqual(self.cmp.dist('abcdefg', 'abcdefg'), 0)
        self.assertEqual(self.cmp.dist('abcdefg', ''), 1)
        self.assertEqual(self.cmp.dist('', 'abcdefg'), 1)

        # https://en.wikipedia.org/wiki/Match_rating_approach
        self.assertAlmostEqual(self.cmp.dist('Byrne', 'Boern'), 1 / 6)
        self.assertAlmostEqual(self.cmp.dist('Smith', 'Smyth'), 1 / 6)
        self.assertAlmostEqual(self.cmp.dist('Catherine', 'Kathryn'), 2 / 6)

        self.assertEqual(self.cmp.dist('ab', 'abcdefgh'), 1)
        self.assertAlmostEqual(self.cmp.dist('ab', 'ac'), 1 / 6)
        self.assertAlmostEqual(self.cmp.dist('abcdefik', 'abcdefgh'), 3 / 6)
        self.assertEqual(self.cmp.dist('xyz', 'abc'), 1)


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_soundex_br.

This module contains unit tests for abydos.phonetic.SoundexBR
"""

import unittest

from abydos.phonetic import SoundexBR


class SoundexBRTestCases(unittest.TestCase):
    """Test SoundexBR functions.

    test cases for abydos.phonetic.SoundexBR
    """

    pa = SoundexBR()

    def test_soundex_br(self):
        """Test abydos.phonetic.SoundexBR."""
        # Base case
        self.assertEqual(self.pa.encode(''), '0000')

        # Examples given at https://github.com/danielmarcelino/SoundexBR
        self.assertEqual(self.pa.encode('Ana Karolina Kuhnen'), 'A526')
        self.assertEqual(self.pa.encode('Ana Carolina Kuhnen'), 'A526')
        self.assertEqual(self.pa.encode('Ana Karolina'), 'A526')
        self.assertEqual(self.pa.encode('João Souza'), 'J220')
        self.assertEqual(self.pa.encode('Dilma Vana Rousseff'), 'D451')
        self.assertEqual(self.pa.encode('Dilma Rousef'), 'D456')
        self.assertEqual(self.pa.encode('Aécio Neves'), 'A251')
        self.assertEqual(self.pa.encode('Aecio Neves'), 'A251')
        self.assertEqual(self.pa.encode('HILBERT'), 'I416')
        self.assertEqual(self.pa.encode('Heilbronn'), 'E416')
        self.assertEqual(self.pa.encode('Gauss'), 'G200')
        self.assertEqual(self.pa.encode('Kant'), 'C530')

        # Tests to complete coverage
        self.assertEqual(self.pa.encode('Wasser'), 'V260')
        self.assertEqual(self.pa.encode('Cici'), 'S200')
        self.assertEqual(self.pa.encode('Gerard'), 'J663')
        self.assertEqual(self.pa.encode('Yglesias'), 'I242')
        self.assertEqual(SoundexBR(zero_pad=False).encode('Cici'), 'S2')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Aecio Neves'), 'AKNP')
        self.assertEqual(self.pa.encode_alpha('HILBERT'), 'ILPR')
        self.assertEqual(self.pa.encode_alpha('Heilbronn'), 'ELPR')
        self.assertEqual(self.pa.encode_alpha('Gauss'), 'GK')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phones.test_phones.

This module contains unit tests for abydos.phones.
"""

import unittest
from math import isnan

from abydos.phones import (
    cmp_features,
    get_feature,
    ipa_to_feature_dicts,
    ipa_to_features,
)


class PhonesTestCases(unittest.TestCase):
    """Test abydos.phones."""

    def test_phones_ipa_to_features(self):
        """Test abydos.phones.ipa_to_features."""
        self.assertEqual(
            ipa_to_features('medçen'),
            [
                2709662981243185770,
                1826957430176000426,
                2783230754501864106,
                2783233463150094762,
                1826957430176000426,
                2711173160463936106,
            ],
        )
        self.assertEqual(
            ipa_to_features('axtuŋ'),
            [
                1826957425952336298,
                2783233462881659306,
                2783230754502126250,
                1825831513894594986,
                2711175868843469418,
            ],
        )
        self.assertEqual(
            ipa_to_features('iç'), [1826957412996131242, 2783233463150094762]
        )
        self.assertEqual(
            ipa_to_features('bakofen'),
            [
                2781720575281113770,
                1826957425952336298,
                2783233462881659562,
                1825831531074464170,
                2781702983095331242,
                1826957430176000426,
                2711173160463936106,
            ],
        )
        self.assertEqual(
            ipa_to_features('dʒuŋel'),
            [
                2783230754501864106,
                2783231556184353178,
                1825831513894594986,
                2711175868843469418,
                1826957430176000426,
                2693158761954453926,
            ],
        )
        self.assertEqual(
            ipa_to_features('kvatʃ'),
            [
                2783233462881659562,
                2781702983095069098,
                1826957425952336298,
                2783230754502126250,
                2783231556184615322,
            ],
        )
        self.assertEqual(
            ipa_to_features('nitʃe'),
            [
                2711173160463936106,
                1826957412996131242,
                2783230754502126250,
                2783231556184615322,
                1826957430176000426,
            ],
        )
        self.assertEqual(
            ipa_to_features('klø'),
            [2783233462881659562, 2693158761954453926, 1825831530269157802],
        )
        self.assertEqual(
            ipa_to_features('kybax'),
            [
                2783233462881659562,
                1825831513089288618,
                2781720575281113770,
                1826957425952336298,
                2783233462881659306,
            ],
        )
        self.assertEqual(
            ipa_to_features('i@c'),
            [1826957412996131242, -1, 2783233463150095018],
        )

    def test_phones_ipa_to_feature_dicts(self):
        """Test abydos.phones.ipa_to_feature_dicts."""
        self.assertEqual(
            ipa_to_feature_dicts('medçen'),
            [
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '+',
                    'approximant': '-',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '-',
                    'high': '0',
                    'low': '0',
                    'front': '0',
                    'back': '0',
                    'tense': '0',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '+',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '-',
                    'low': '-',
                    'front': '+',
                    'back': '-',
                    'tense': '+',
                    'pharyngeal': '+',
                    'atr': '+',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '+',
                    'anterior': '+',
                    'distributed': '-',
                    'dorsal': '-',
                    'high': '0',
                    'low': '0',
                    'front': '0',
                    'back': '0',
                    'tense': '0',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '-',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '-',
                    'low': '-',
                    'front': '+',
                    'back': '-',
                    'tense': '+',
                    'pharyngeal': '+',
                    'atr': '+',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '+',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '+',
                    'anterior': '+',
                    'distributed': '-',
                    'dorsal': '-',
                    'high': '0',
                    'low': '0',
                    'front': '0',
                    'back': '0',
                    'tense': '0',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '+',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
            ],
        )
        self.assertEqual(
            ipa_to_feature_dicts('axtuŋ'),
            [
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '-',
                    'low': '+',
                    'front': '+',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '+',
                    'atr': '-',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '+',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '-',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '+',
                    'anterior': '+',
                    'distributed': '-',
                    'dorsal': '-',
                    'high': '0',
                    'low': '0',
                    'front': '0',
                    'back': '0',
                    'tense': '0',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '-',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '+',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '+',
                    'tense': '+',
                    'pharyngeal': '+',
                    'atr': '+',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '+',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '+',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '+',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
            ],
        )
        self.assertEqual(
            ipa_to_feature_dicts('iç'),
            [
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '+',
                    'back': '-',
                    'tense': '+',
                    'pharyngeal': '+',
                    'atr': '+',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '-',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
            ],
        )
        self.assertEqual(
            ipa_to_feature_dicts('ʤɪn'),
            [
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '+',
                    'anterior': '-',
                    'distributed': '+',
                    'dorsal': '+',
                    'high': '-',
                    'low': '-',
                    'front': '-',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+/-',
                    'nasal': '-',
                    'strident': '+',
                    'lateral': '-',
                    'delayed_release': '+',
                },
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '+',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '+',
                    'atr': '-',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '+',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '+',
                    'anterior': '+',
                    'distributed': '-',
                    'dorsal': '-',
                    'high': '0',
                    'low': '0',
                    'front': '0',
                    'back': '0',
                    'tense': '0',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '+',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
            ],
        )
        self.assertEqual(
            ipa_to_feature_dicts('i@c'),
            [
                {
                    'syllabic': '+',
                    'consonantal': '-',
                    'sonorant': '+',
                    'approximant': '+',
                    'labial': '+',
                    'round': '-',
                    'protruded': '-',
                    'compressed': '-',
                    'labiodental': '-',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '+',
                    'back': '-',
                    'tense': '+',
                    'pharyngeal': '+',
                    'atr': '+',
                    'rtr': '-',
                    'voice': '+',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '+',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
                {},
                {
                    'syllabic': '-',
                    'consonantal': '+',
                    'sonorant': '-',
                    'approximant': '-',
                    'labial': '-',
                    'round': '0',
                    'protruded': '0',
                    'compressed': '0',
                    'labiodental': '0',
                    'coronal': '-',
                    'anterior': '0',
                    'distributed': '0',
                    'dorsal': '+',
                    'high': '+',
                    'low': '-',
                    'front': '-',
                    'back': '-',
                    'tense': '-',
                    'pharyngeal': '-',
                    'atr': '0',
                    'rtr': '0',
                    'voice': '-',
                    'spread_glottis': '-',
                    'constricted_glottis': '-',
                    'glottalic_suction': '-',
                    'velaric_suction': '-',
                    'continuant': '-',
                    'nasal': '-',
                    'strident': '-',
                    'lateral': '-',
                    'delayed_release': '-',
                },
            ],
        )

    def test_phones_get_feature(self):
        """Test abydos.phones.get_feature."""
        self.assertEqual(
            get_feature(ipa_to_features('medçen'), 'nasal'),
            [1, -1, -1, -1, -1, 1],
        )
        self.assertRaises(
            AttributeError, get_feature, ipa_to_features('medçen'), 'vocalic'
        )

        self.assertEqual(
            get_feature(ipa_to_features('nitʃe'), 'nasal'), [1, -1, -1, -1, -1]
        )
        self.assertEqual(
            get_feature(ipa_to_features('nitʃe'), 'strident'),
            [-1, -1, -1, 1, -1],
        )
        self.assertEqual(
            get_feature(ipa_to_features('nitʃe'), 'syllabic'),
            [-1, 1, -1, -1, 1],
        )
        self.assertEqual(
            get_feature(ipa_to_features('nitʃe'), 'continuant'),
            [-1, 1, -1, 1, 1],
        )

        self.assertEqual(
            get_feature(ipa_to_features('nit͡ʃe'), 'nasal'), [1, -1, -1, -1]
        )
        self.assertEqual(
            get_feature(ipa_to_features('nit͡ʃe'), 'strident'), [-1, -1, 1, -1]
        )
        self.assertEqual(
            get_feature(ipa_to_features('nit͡ʃe'), 'syllabic'), [-1, 1, -1, 1]
        )
        self.assertEqual(
            get_feature(ipa_to_features('nit͡ʃe'), 'continuant'), [-1, 1, 2, 1]
        )

        self.assertEqual(
            get_feature(ipa_to_features('løvenbroy'), 'atr'),
            [0, 1, 0, 1, 0, 0, 0, 1, 1],
        )
        self.assertNotEqual(
            get_feature(ipa_to_features('i@c'), 'syllabic'),
            [1, float('NaN'), -1],
        )
        self.assertTrue(
            isnan(get_feature(ipa_to_features('i@c'), 'syllabic')[1])
        )

    def test_phones_cmp_features(self):
        """Test abydos.phones.cmp_features."""
        # # negatives
        self.assertEqual(cmp_features(-1, 1826957425952336298), 0)
        self.assertEqual(cmp_features(1826957425952336298, -1), 0)
        self.assertEqual(cmp_features(-1, -1), 0)
        # # equals
        self.assertEqual(cmp_features(0, 0), 1)
        self.assertEqual(
            cmp_features(1826957425952336298, 1826957425952336298), 1
        )

        # # unequals
        # pre-calc everything
        cced = ipa_to_features('ç')[0]
        esh = ipa_to_features('ʃ')[0]
        tesh = ipa_to_features('t͡ʃ')[0]

        cmp_cced_tesh = cmp_features(cced, tesh)
        cmp_cced_esh = cmp_features(cced, esh)
        cmp_esh_tesh = cmp_features(esh, tesh)

        cmp_tesh_cced = cmp_features(tesh, cced)
        cmp_esh_cced = cmp_features(esh, cced)
        cmp_tesh_esh = cmp_features(tesh, esh)

        # check symmetric equality
        self.assertEqual(cmp_cced_tesh, cmp_tesh_cced)
        self.assertEqual(cmp_cced_esh, cmp_esh_cced)
        self.assertEqual(cmp_esh_tesh, cmp_tesh_esh)

        # check that they're all greater than 0
        self.assertGreater(cmp_cced_tesh, 0)
        self.assertGreater(cmp_cced_esh, 0)
        self.assertGreater(cmp_esh_tesh, 0)

        # check that they're all less than 1
        self.assertLess(cmp_cced_tesh, 1)
        self.assertLess(cmp_cced_esh, 1)
        self.assertLess(cmp_esh_tesh, 1)

        # ʃ and t͡ʃ should be more similar than either of these and ç
        self.assertGreater(cmp_esh_tesh, cmp_cced_tesh)
        self.assertGreater(cmp_esh_tesh, cmp_cced_esh)

        # weight modification
        self.assertEqual(cmp_features(cced, esh), 0.8709677419354839)
        self.assertEqual(cmp_features(cced, esh, {'syllabic': 1}), 1)
        self.assertEqual(
            cmp_features(cced, esh, [1, 1, 1]), 0.6666666666666667
        )
        with self.assertRaises(TypeError):
            cmp_features(cced, esh, 10)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_prefix.

This module contains unit tests for abydos.distance.Prefix
"""

import unittest

from abydos.distance import Prefix


class PrefixTestCases(unittest.TestCase):
    """Test prefix similarity functions.

    abydos.distance.Prefix
    """

    cmp = Prefix()

    def test_prefix_sim(self):
        """Test abydos.distance.Prefix.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('a', ''), 0)
        self.assertEqual(self.cmp.sim('', 'a'), 0)
        self.assertEqual(self.cmp.sim('a', 'a'), 1)
        self.assertEqual(self.cmp.sim('ax', 'a'), 1)
        self.assertEqual(self.cmp.sim('axx', 'a'), 1)
        self.assertEqual(self.cmp.sim('ax', 'ay'), 1 / 2)
        self.assertEqual(self.cmp.sim('a', 'ay'), 1)
        self.assertEqual(self.cmp.sim('a', 'ayy'), 1)
        self.assertEqual(self.cmp.sim('ax', 'ay'), 1 / 2)
        self.assertEqual(self.cmp.sim('a', 'y'), 0)
        self.assertEqual(self.cmp.sim('y', 'a'), 0)
        self.assertEqual(self.cmp.sim('aaax', 'aaa'), 1)
        self.assertAlmostEqual(self.cmp.sim('axxx', 'aaa'), 1 / 3)
        self.assertEqual(self.cmp.sim('aaxx', 'aayy'), 1 / 2)
        self.assertEqual(self.cmp.sim('xxaa', 'yyaa'), 0)
        self.assertAlmostEqual(self.cmp.sim('aaxxx', 'aay'), 2 / 3)
        self.assertEqual(self.cmp.sim('aaxxxx', 'aayyy'), 2 / 5)
        self.assertEqual(self.cmp.sim('xa', 'a'), 0)
        self.assertEqual(self.cmp.sim('xxa', 'a'), 0)
        self.assertEqual(self.cmp.sim('xa', 'ya'), 0)
        self.assertEqual(self.cmp.sim('a', 'ya'), 0)
        self.assertEqual(self.cmp.sim('a', 'yya'), 0)
        self.assertEqual(self.cmp.sim('xa', 'ya'), 0)
        self.assertEqual(self.cmp.sim('xaaa', 'aaa'), 0)
        self.assertEqual(self.cmp.sim('xxxa', 'aaa'), 0)
        self.assertEqual(self.cmp.sim('xxxaa', 'yaa'), 0)
        self.assertEqual(self.cmp.sim('xxxxaa', 'yyyaa'), 0)

    def test_prefix_dist(self):
        """Test abydos.distance.Prefix.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('a', ''), 1)
        self.assertEqual(self.cmp.dist('', 'a'), 1)
        self.assertEqual(self.cmp.dist('a', 'a'), 0)
        self.assertEqual(self.cmp.dist('ax', 'a'), 0)
        self.assertEqual(self.cmp.dist('axx', 'a'), 0)
        self.assertEqual(self.cmp.dist('ax', 'ay'), 1 / 2)
        self.assertEqual(self.cmp.dist('a', 'ay'), 0)
        self.assertEqual(self.cmp.dist('a', 'ayy'), 0)
        self.assertEqual(self.cmp.dist('ax', 'ay'), 1 / 2)
        self.assertEqual(self.cmp.dist('a', 'y'), 1)
        self.assertEqual(self.cmp.dist('y', 'a'), 1)
        self.assertEqual(self.cmp.dist('aaax', 'aaa'), 0)
        self.assertAlmostEqual(self.cmp.dist('axxx', 'aaa'), 2 / 3)
        self.assertEqual(self.cmp.dist('aaxx', 'aayy'), 1 / 2)
        self.assertEqual(self.cmp.dist('xxaa', 'yyaa'), 1)
        self.assertAlmostEqual(self.cmp.dist('aaxxx', 'aay'), 1 / 3)
        self.assertEqual(self.cmp.dist('aaxxxx', 'aayyy'), 3 / 5)
        self.assertEqual(self.cmp.dist('xa', 'a'), 1)
        self.assertEqual(self.cmp.dist('xxa', 'a'), 1)
        self.assertEqual(self.cmp.dist('xa', 'ya'), 1)
        self.assertEqual(self.cmp.dist('a', 'ya'), 1)
        self.assertEqual(self.cmp.dist('a', 'yya'), 1)
        self.assertEqual(self.cmp.dist('xa', 'ya'), 1)
        self.assertEqual(self.cmp.dist('xaaa', 'aaa'), 1)
        self.assertEqual(self.cmp.dist('xxxa', 'aaa'), 1)
        self.assertEqual(self.cmp.dist('xxxaa', 'yaa'), 1)
        self.assertEqual(self.cmp.dist('xxxxaa', 'yyyaa'), 1)


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_ONCA.

This module contains unit tests for abydos.phonetic.ONCA
"""

import unittest

from abydos.phonetic import ONCA


class ONCATestCases(unittest.TestCase):
    """Test ONCA functions.

    test cases for abydos.phonetic.ONCA
    """

    pa = ONCA()

    def test_onca(self):
        """Test abydos.phonetic.ONCA."""
        # https://nces.ed.gov/FCSM/pdf/RLT97.pdf
        self.assertEqual(self.pa.encode('HALL'), 'H400')
        self.assertEqual(self.pa.encode('SMITH'), 'S530')

        # http://nchod.uhce.ox.ac.uk/NCHOD%20Oxford%20E5%20Report%201st%20Feb_VerAM2.pdf
        self.assertEqual(self.pa.encode('HAWTON'), 'H350')
        self.assertEqual(self.pa.encode('HORTON'), 'H635')
        self.assertEqual(self.pa.encode('HOUGHTON'), 'H235')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('HALL'), 'HL')
        self.assertEqual(self.pa.encode_alpha('SMITH'), 'SNT')
        self.assertEqual(self.pa.encode_alpha('HOUGHTON'), 'HKTN')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_string_fingerprint.

This module contains unit tests for abydos.fingerprint.String
"""

import unittest

from abydos.fingerprint import String


class StringTestCases(unittest.TestCase):
    """Test string fingerprint functions.

    abydos.fingerprint.String
    """

    fp = String()

    _testset = (
        'À noite, vovô Kowalsky vê o ímã cair no pé do pingüim \
queixoso e vovó põe açúcar no chá de tâmaras do jabuti feliz.',
    )
    _anssetw = (
        'a acucar cair cha de do e feliz ima jabuti kowalsky no noite \
o pe pinguim poe queixoso tamaras ve vovo',
    )

    def test_string_fingerprint(self):
        """Test abydos.fingerprint.String."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '')

        for i in range(len(self._testset)):
            self.assertEqual(
                self.fp.fingerprint(self._testset[i]), self._anssetw[i]
            )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_sift4_extended.

This module contains unit tests for abydos.distance.Sift4Extended
"""

import unittest

from abydos.distance import Sift4Extended
from abydos.tokenizer import QGrams


class Sift4ExtendedTestCases(unittest.TestCase):
    """Test Sift4Extended functions.

    abydos.distance.Sift4Extended
    """

    ltamc = Sift4Extended.longer_transpositions_are_more_costly

    cmp = Sift4Extended()
    cmp_kwargs = Sift4Extended(
        tokenizer=QGrams(qval=2),
        token_matcher=Sift4Extended.sift4_token_matcher,
        matching_evaluator=Sift4Extended.sift4_matching_evaluator,
        local_length_evaluator=Sift4Extended.reward_length_evaluator,
        transposition_cost_evaluator=ltamc,
        transpositions_evaluator=lambda lcss, trans: lcss - trans,
    )
    cmp_kwargs2 = Sift4Extended(
        local_length_evaluator=Sift4Extended.reward_length_evaluator_exp
    )
    cmp_md = Sift4Extended(max_distance=3)

    def test_sift4_extended_dist_abs(self):
        """Test abydos.distance.Sift4Extended.dist_abs."""
        # Base cases
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 3)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 3)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0)
        self.assertEqual(self.cmp.dist_abs('abcd', 'efgh'), 4)

        self.assertAlmostEqual(self.cmp.dist_abs('Nigel', 'Niall'), 2)
        self.assertAlmostEqual(self.cmp.dist_abs('Niall', 'Nigel'), 2)
        self.assertAlmostEqual(self.cmp.dist_abs('Colin', 'Coiln'), 1)
        self.assertAlmostEqual(self.cmp.dist_abs('Coiln', 'Colin'), 1)
        self.assertAlmostEqual(self.cmp.dist_abs('ATCAACGAGT', 'AACGATTAG'), 4)

        self.assertEqual(self.cmp_kwargs.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_kwargs.dist_abs('a', ''), 2)
        self.assertEqual(self.cmp_kwargs.dist_abs('', 'a'), 2)
        self.assertEqual(self.cmp_kwargs.dist_abs('abc', ''), 4)
        self.assertEqual(self.cmp_kwargs.dist_abs('', 'abc'), 4)
        self.assertEqual(self.cmp_kwargs.dist_abs('abc', 'abc'), -1)
        self.assertEqual(self.cmp_kwargs.dist_abs('abcd', 'efgh'), -2)

        self.assertAlmostEqual(self.cmp_kwargs.dist_abs('Nigel', 'Niall'), 1)
        self.assertAlmostEqual(self.cmp_kwargs.dist_abs('Niall', 'Nigel'), 1)
        self.assertAlmostEqual(self.cmp_kwargs.dist_abs('Colin', 'Coiln'), 1)
        self.assertAlmostEqual(self.cmp_kwargs.dist_abs('Coiln', 'Colin'), 1)
        self.assertAlmostEqual(
            self.cmp_kwargs.dist_abs('ATCAACGAGT', 'AACGATTAG'), 2
        )

        self.assertEqual(self.cmp_kwargs2.dist_abs('abc', 'abc'), 0)
        self.assertEqual(self.cmp_kwargs2.dist_abs('abcd', 'efgh'), 8)

        self.assertAlmostEqual(self.cmp_kwargs2.dist_abs('Nigel', 'Niall'), 7)
        self.assertAlmostEqual(self.cmp_kwargs2.dist_abs('Niall', 'Nigel'), 7)
        self.assertAlmostEqual(self.cmp_kwargs2.dist_abs('Colin', 'Coiln'), 6)
        self.assertAlmostEqual(self.cmp_kwargs2.dist_abs('Coiln', 'Colin'), 6)
        self.assertAlmostEqual(
            self.cmp_kwargs2.dist_abs('ATCAACGAGT', 'AACGATTAG'), 25
        )

        # coverage completion
        self.assertAlmostEqual(
            self.cmp_kwargs.dist_abs('beaurocracy', 'bureaucracy'), 3
        )
        self.assertAlmostEqual(
            self.cmp_md.dist_abs('beaurocratically', 'bureaucracy'), 3
        )
        self.assertAlmostEqual(
            self.cmp_md.dist_abs('bureaucracy', 'bureaucracy'), 3
        )


if __name__ == '__main__':
    unittest.main()
